C$DEBUG
C     Program PKREF.FOR
C     PKREF.for(Version 05/2000) for MS FORTRAN under DOS or Windows NT
C     Modified by J.E.Bertie  in April 2000 from Zhida Lan's
C     PKREF.for(Version 01/1996)
C     Modified by J.E.Bertie in May 2005 to read/write type 2 SPC files
C     as well as the original type 1 SPC files.    C	The distinction 
C     between reading old or new SPC files is made in READSC which is
C     in BSPCRW.  The program writes the same type of SPC file as it
C     reads (With modification this can be changed).
C
C     Written in the Department of Chemistry, University of Alberta,
C     Edmonton, Alberta, Canada
C
C     THIS PROGRAM  uses subroutine pATR2PNK to calculate the phase
C     shift on reflection and n and k spectra of the sample from an
C     experimental pATR spectrum that was measured in a CIRCLE CELL.
C     In the CIRCLE Cell, the angle of incidence is 45 deg and the
C     reflectance is 0.5*(Rs**REF + Rp**REF) = 0.5*(Rs**REF + Rs**2REF),
C     where REF is the effective number of reflections
C
c PKREF.FOR WAS BASED ON ZHANG'S KREF.FOR AND LAN'S MREPH3.FOR 
c WITH IMPROVED ITERATIVE PROCEDURE due to URBAN AND HUANG (SEE REPORT
C "REFLECTANCE TO PHASE SHIFT METHOD TO REFINE REFLECTANCE SPECTRA"
C WRITTEN BY ZHIDA LAN), 
C  See also Bertie and Lan, J. Chem. Phys. 103, 10152 -10161 (1995))
C
C  Input is from file PKREF.INP
C  Note that all filenames must be in quotation marks, must be complete
C  with extension, with a maximum of 12 characters including the period
C
C LINE 1: Either 'KRK' or 'RKR'. The accepted values, in quotes, of the 
C         3-byte variable KRK.
C     IF KRK='KRK', the program calculates the initial phase shift
C     spectrum from the pATR spectrum via the KK transform described by 
C     J.E. Bertie and Z. Lan in J. Chem. Phys. 105, 8502 (1996).
C     IF KRK='RKR' the program reads this initial phase shift spectrum
C     In this mode the program does the same as Z. Lan's original PKREF
C     but with input in different order.
C     In both cases the program refines the initial spectra following
C     Urban and Huang, Appl. Spectrosc. 46, 1666 (1992), modified as
C     described in Bertie and Lan above.
C
C LINE 2:	Three Filenames:
C     pATR.SPC:  	pATR spectrum file  
C     nROD.ASC:  	ASCII file with 2-column n~ - n data for ATR rod
C     pkrefOUT.TXT : File for text output.
C
C LINE 3:	Four *.SPC filenames:
C     First for the final k spectrum
C     Second for the final n spectrum
C     Third for the final phase shift spectrum
C     Fourth for the final calculated pATR spectrum
C
C LINE 4:	3 Real numbers:  NINF, A2 and A4, from 
C     n = NINF + A2n~2 + A4n~4 for visible n
C
C LINE 5: Three numbers: REF, NCB,WNCRITIC
C     REF, a REAL number, is the effective number of reflections in
C        the CIRCLE cell.
C     NCB, an INTEGER number, the maximum number of iterations.
C        Usually set to 250 to ensure convergence.
C     WNCRITIC, a REAL number is 0.0 if a the K-K transform is to use
C        every data point at each wavenumber.
C     OR WNCRITIC is the value of the critical wavenumber if the
C        spectrum above wavenumber WNCRITIC changes so slowly that it
C        is sufficient to use every 100th data point above WNCRITIC
C        and to omit these points from the KK transform for
C        wavenumbers below WNCRITIC. Use of this approximation is much
C        faster. Note that PKREF automatically reverts to the full KK
C        when close to the iteration limit, (see FSUM1 below).
C
C LINE 6: 4 entries, each one 'YES' or 'NO'
C     YN 'YES' if *.SPC files are wanted of the k, n, and phase shift
C        spectra in each cycle.  'NO' if they are not wanted.  If you
C        give 'YES' here, you must put at the end of the input one
C        line for each cycle, except the first, giving the *.SPC
C        filenames for the k,n,and phase shift spectra in that order.
C        The filenames for the first cycle are controlled by INITOUT.
C        N.B. The program automatically writes the k, nand phase shift
C        spectra every 5th. cycle, but overwrites the file each time so
C        only the last "5th-cycle" files survive.
C     INITOUT, enter 'YES' if .SPC output files are wanted of the
C        initial k, n, and, if KRK= 'KRK', phase shift spectra.
C        Enter 'NO' otherwise.  'NO' is the usual entry but this
C        provides a useful check.
C     NRODSPC	'YES' if an .SPC output file is wanted of the
C        interpolated spectrum of n of the ATR rod.
C     BIGSPC  'YES' if an .SPC output file is wanted of the extended
C        file input to the KK transform.  This extended input results
C        from NREST in lines 8 and 12.  The output files are called
C        BIGHLnRs.SPC, Bigk.SPC, APDHLnRs.SPC, or kAFTAPND.SPC.
C
C LINE 7	ONLY NEEDED IF INITOUT='YES' in Line 6
C     Two *.SPC filenames.  
C     First, for the initial k
C     Second for initial n
C
C LINE 8	ONLY NEEDED IF KRK = 'KRK' in line 1:  NREST and RLAST
C     NREST, an INTEGER number that controls how the program deals
C        with lack of reflectance data to low wavenumber of the
C        measured spectrum.
C        If NREST=0 the data is used as supplied.  
C        If NREST>0 NREST points are added to the low wavenumber end
C        with the same spacing.  If NREST points extends the file
C        below 0 cm-1 the data is extended only to 0 cm-1.
C        If NREST<0 an .SPC file is read in and appended to the 
C        0.5*ln(Rs) array before the KK transform.  This file must be
C        a file of  0.5*ln(Rs)
C     RLAST, a REAL number; the value of Ln(Rs) wanted at the last 
C        extended point.  The file is extended linearly from the last
C        measured point to RLAST at 0 cm^-1
C        If the extension ends at 0 cm^-1, RLAST must be zero.
C     RLAST must be given even if NREST is zero or negative.
C
C LINE 9	ONLY NEEDED IF KRK = 'KRK' in line 1 AND INITOUT = 'YES'
C     in line 7
C     One *.SPC filename for the initial Phase shift spectrum.
C
C LINE 10	ONLY NEEDED IF KRK='KRK' and NREST in line 7 is negative.
C     One *.SPC filename of the 0.5*ln(Rs) spectrum to append to the
C        0.5*ln(Rs)array before the KK transform.
C
C LINE 11	ONLY Needed if KRK='RKR' in Line 1.
C     One *.SPC filename for the initial phase shift spectrum.
C
C LINE 12: 2 numbers NREST and RLAST.  These have the same meaning
C     as in line 8 except that now they refer to the KK transform 
C     from k to n that occurs in each cycle.
C
C LINE 13: ONLY NEEDED IF NREST in line 12 is negative.
C     One *.SPC filename for the k spectrum to append to the k
C        array before the k to n KK transforms.
C
C LINE 14: If WNCRITIC <0 or =0, two REAL numbers LSUM and LRMS
C          If WNCRITIC >0, three REAL numbers LSUM, LRMS and FSUM1.
C     LSUM is the value of the sum squared deviations between the 
C        calculated and experimental pATR spectra that must be reached
C        before convergence is declared. 
C        LSUM is usually set at 0.00005.
C     LRMS is the value of the rms percent deviation between the 
C        calculated and experimental pATR spectra that must be reached
C        before convergence is declared. 
C        LRMS is usually set at 0.1.
C     FSUM must be greater than LSUM. This value is used only if
C        the value of WNCRITIC is greater than zero.  Then PKREF uses
C        the approximate KK when LSUM > FSUM1 and switches to the full
C        KK when LSUM <FSUM1. 
C        FSUM1 is usually set at 0.0005
C
C LINES 15 to 15+NCB: three *.SPC filenames per line, one line for 
C     each cycle.  K?.SPC, n?.SPC and pATRC?.SPC for the output of
C     the k, n, and calculated pATR spectra at the end of each cycle
C    J.E. Bertie   May 4 2000
C
C
      PARAMETER (NMAX=9336)
	LOGICAL       OUTC
      INTEGER*1     XT,YT
	INTEGER*2	  KC,INCHAN
      INTEGER*4     NFIX,NSPC,ier    
      CHARACTER*3   KRK,INITOUT,NRODSPC,BIGSPC,YN
      CHARACTER*192 COMM
      CHARACTER*12  pATRname,nRODname,pkrefOUT,Kinit,Ninit,PHASinit
      CHARACTER*12  EXTND4KK,Kfinal,Nfinal,pATRCfin,PHASEfin
      CHARACTER*12  NNAME,KNAME,pATRCNAM,BIGNAME,APNDNAME
      REAL*4        LSUM, LRMS,NROD,IMAGK
      DIMENSION     NROD(NMAX),IMAGK(NMAX),PATR(NMAX),PHASE(NMAX)
      DIMENSION     x(NMAX),x2(NMAX),REALN(NMAX),PATRC(9336)
      COMMON/CHAR/EXTND4KK,COMM,NRODSPC,BIGSPC,BIGNAME,APNDNAME
      COMMON/JEB1/  NSPC,XS,XE,RES,iread,NTOT
	COMMON/JEB2/  WNCRITIC,NREST,RLAST,RNINF,REF
	COMMON/JEB3/  XT,YT
	COMMON/textIO/ KC
	COMMON/FOR_READ/NPTLIMIT
      
	NPTLIMIT=NMAX
      YN='NO'
      OUTC=.FALSE.
      INITOUT='NO'

      write(*,*)
      WRITE(*,*)' PKREF, V.03/2005 begins '
      write(*,*)
	write(*,950)NPTLIMIT
  950 FORMAT(/,'  Input spectral files are limited to',I6,' points',/,
     1'  No output is produced from larger files',/) 

      
	CALL GETCHAN(INCHAN,IER)
      OPEN(UNIT=INCHAN,FILE='PKREF.INP',STATUS='OLD')

      READ(INCHAN,*) KRK
      READ(INCHAN,*) pATRname,nRODname,pkrefOUT
	
      CALL GETCHAN(KC,IER)
      OPEN(UNIT=KC,FILE=pkrefOUT,STATUS='UNKNOWN')
      WRITE(KC,*)' PKREF, V.03/2005 begins '
	WRITE(KC,*)

      READ(INCHAN,*) KFINAL,Nfinal,PHASEfin,pATRCfin
C     The next 3 parameters are from fitting n at visible wavenumbers
C     to n(v) = RNINF+A2*v^2+A4*v^4. They give n due only to electronic
C     polarization(JEB and Z.LAN, J.Chem.Phys. 103, 10152 (1995))
      READ(INCHAN,*) RNINF,A2,A4
      READ(INCHAN,*) REF,NCB, WNCRITIC
      READ(INCHAN,*) YN,INITOUT,NRODSPC,BIGSPC
	      
      IF (YN.EQ.'YES') OUTC=.TRUE.
	IF (INITOUT .EQ.'YES')READ(INCHAN,*) Kinit,Ninit
	IF (KRK .EQ. 'KRK')THEN
          READ(INCHAN,*)NREST,RLAST
	    IF (INITOUT .EQ.'YES') READ(INCHAN,*) PHASinit
          IF (NREST .LT.0) READ(INCHAN,*) EXTND4KK
      ENDIF

      WRITE(*,*)' READING pATR'
      CALL READSC(pATRname,PATR,NFIX,XS,XE,XT,YT,RES,COMM,IER)
C     CHECK THE NUMBER OF POINTS AND STOP IF TOO LARGE
         IF(NFIX.GT.NPTLIMIT) THEN
           WRITE(*,920)pATRname,NPTLIMIT
  920 FORMAT(' PKREF:File ',A12,':Number of Points exceeds array size'
     1 1X,I5,/,' No output created',/)
           GO TO 500
	   ENDIF

      WRITE(KC,*)' The program is in mode ', KRK
      WRITE(KC,*) 'The pATR spectrum is from file ', pATRname
      WRITE(KC,100) pATRname,NFIX,XS,XE
      WRITE(KC,*) 'The n of rod file is from file ', nRODname
      WRITE(KC,*) 'The text output is in file ', pkrefOUT
      WRITE(KC,'(3(a,f7.4))')' n(infinity) = ', RNINF,',  A2 = ',A2,
     +',  A4= ',A4
      WRITE(KC,'(a,f7.4)')' number of reflections (NRF) = ',REF
	IF (WNCRITIC .NE.0.0) THEN
          WRITE(KC,'(a,f8.2,a)')' Use approx KK above ',WNCRITIC,' cm-1'
	ENDIF
	IF (INITOUT .EQ.'YES') THEN
           WRITE(KC,*)' Will write initial k and n spectra to ',
     +                  Kinit,' and ',Ninit
	ELSE
	       WRITE(KC,*)' Will not write initial k and n spectra'
      ENDIF

	IF (OUTC) WRITE(KC,*)' Will Write k,n and phase files each cycle'
	
      IF (KRK .EQ. 'KRK')THEN
	    IF (INITOUT .EQ.'YES') THEN
	       WRITE(KC,*)' Will write initial phase spectrum to',PHASinit
     		ELSE
	       WRITE(KC,*)' Will not write initial phase spectrum'
          ENDIF

	    IF (NREST .GE.0) THEN
            WRITE(KC,'(i5,a,a,/,a,f7.4)')NREST,'pts will be added ',
     +            'to the 0.5*ln(Rs) spectrum at low nu;',
     +           ' The value at the last point =',RLAST
         EXTND4KK='NOAPPEND'
          ELSE
         WRITE(KC,*)'The added 0.5*ln(Rs) spectrum below the ',
     +              'low-wavenumber limit is from  ',EXTND4KK
	    ENDIF
      ENDIF

      NSPC=NFIX
      iread=0
      dx=(xs-xe)/float(NSPC-1)
      DO i=1,NMAX
         xtemp=xs-dx*float(i-1)
         x(i)=xtemp
         x2(i)=xtemp*xtemp
      END DO
      WRITE(*,*)' INTERPOLATING N(ROD)'
      CALL MAKENROD(nRODname,NROD)
C     IF KRK='KRK',calculate phase shift, n and k
      IF (KRK .EQ. 'KRK') THEN
C     Calculate RNSNMAX, n of sample at highest wavenumber, XS
        RNSNMAX=RNINF+A2*X2(1)+A4*X2(1)*X2(1)
C     Set APNDNAME and BIGNAME for 0.5LnRs spectra
      APNDNAME='APDHLnRs.SPC'
      BIGNAME='BIGHLnRs.SPC'
	CALL pATR2PH(PATR,NROD,IMAGK,PHASE,X,X2,RNSNMAX)
C     On return, phase shifts are in PHASE and 0.5Ln(Rs) is in IMAGK
C     IF INITOUT='YES' write the phase spectrum to disk
	  IF (INITOUT .EQ.'YES') THEN
	  CALL WRITESC(PHASinit,PHASE,NSPC,XS,XE,XT,YT,RES,COMM,IER)
	  WRITE(KC,*)' MAIN: written initial phase to ',PHASinit
	  ENDIF
      ENDIF

C     If KRK=RKR Read initial phase shift file name and contents
      IF (KRK .EQ. 'RKR') THEN
          READ(INCHAN,*) PHASinit
	    WRITE(KC,*) 'The PHASE spectrum is from file ',PHASinit
          CALL READSC(PHASINIT,PHASE,NFIX2,XS2,XE2,XT,YT,RES,COMM,IER)

C     Check that XS=XS2, XE=XE2 and spacing is the same in both files
          IF (XS .NE. XS2) THEN
            WRITE(*,*) '  Starting wavenumbers differ in pATR and',
     +                 ' Phase files'
            WRITE(*,*) '  Program quits for you to correct input'
            GO TO 500
          ELSE IF (XE .NE. XE2)THEN
            WRITE(*,*) '  Lowest wavenumbers differ in pATR and',
     +                 ' Phase files'
            WRITE(*,*) '  Program quits for you to correct input'
            GO TO 500
          ELSE IF (NFIX .NE.NFIX2) THEN
            WRITE(*,*) '  Number of points differ in pATR and',
     +                 ' Phase files'
            WRITE(*,*) '  Program quits for you to correct input'
            GO TO 500
          ENDIF
C     If files are compatible report PHASINIT parameters
          WRITE(KC,100) PHASINIT,NFIX,XS,XE
      ENDIF

C     Now read NREST, RLAST, and possibly EXTND4KK, for k to n KKs
      READ(INCHAN,*) NREST,RLAST
      IF (NREST .LT.0) THEN
          READ(INCHAN,*) EXTND4KK
          WRITE(KC,*)'The added k spectrum below the low-wavenumber',
     +              ' limit is from  ',EXTND4KK
      ELSE
          WRITE(KC,'(i5,a,a,/,a,f7.4)')NREST,' pts will be added to',
     +        ' the k spectrum at low nu;',
     +        ' the value at the last point will be ',RLAST
          EXTND4KK='NOappendK'
      END IF
      WRITE(KC,*)
C     Read refinement limits and switch-point from approx.to full KK
      IF (wncritic.le.0) then
          wncritic=0.0
          READ(INCHAN,*) LSUM,LRMS
      ELSE
          READ(INCHAN,*) lsum,lrms,fsum1
      ENDIF

      WRITE(KC,*)
            WRITE(KC,'(a,f9.7,a,/,a,f7.5,a)') 
     +  ' Convergence limits are set to ',lsum,
     +  ' for sum squared deviations and ',
     +  '                               ',lrms,
     +  '   for RMS percent deviation'
      WRITE(KC,*)

      if (wncritic.gt.0) then 
	  WRITE(KC,'(a,/,a,f9.7)')
     +    ' The program will switch to the full KK',
     +    ' when the sum of squared deviation reaches ',fsum1
	  WRITE(KC,*)
      endif

      WRITE(KC,*)'The final k spectrum will be written to ',Kfinal
      WRITE(KC,*)'The final n spectrum will be written to ',Nfinal
      WRITE(KC,*)'The final pATRC spectrum will be written to ',pATRCfin
      WRITE(KC,*)'The final PHASE SHIFT spectrum is output to ',
     + PHASEfin
      WRITE(KC,*)

C     Set BIGNAME for k
      APNDNAME='kAFTAPND.SPC'
      BIGNAME='BIGk.SPC'
C     Reset iread for the k to n transforms
      IREAD=0

C  THE ITERATION IS from here to statement 101
      DO 101 INCB=1,NCB
      write(*,*)
      WRITE(*,'(2X,A,I4,A,I4)') 'CYCLE: ',INCB,'   OF ',NCB
	WRITE(KC,*)
      WRITE(KC,'(2X,A,I4,A,I4)') 'CYCLE: ',INCB,'   OF ',NCB

C Calculate k from PHASE and experimental pATR
      CALL RPH2K(PATR,NROD,PHASE,IMAGK)
C     IMAGK now contains k

C     Do k to n KK transform.
      CALL LZKKTSUB(RNINF,WNCRITIC,NREST,RLAST,IMAGK,REALN,x,x2)

C  Correct n for variation of n-electronic through IR
      IF(A2.NE.0.0) THEN
          DO I=1,NSPC
          REALN(I)=REALN(I)+A2*X2(I)+A4*X2(I)*X2(I)
          END DO
      END IF

C     The first time through the loop write the initial k and n to disk
C     IF INITOUT='YES'
      IF (INCB .eq. 1) THEN
	    IF (INITOUT .EQ.'YES') THEN
              CALL WRITESC(Kinit,IMAGK,NSPC,XS,XE,XT,YT,RES,COMM,IER)
              WRITE(KC,*)' MAIN: written initial k to ',Kinit
              CALL WRITESC(Ninit,REALN,NSPC,XS,XE,XT,YT,RES,COMM,IER)
              WRITE(KC,*)' MAIN: written initial n to ',ninit
		ENDIF
	ENDIF	

C     Calculate pATR spectrum into PATRC from n,k and nROD
      CALL REFLCALN(PATRC,REALN,IMAGK,NROD)

C  SUBROUTINE STATS CALCULATES SUMSD AND RMSD
      CALL STATS(NSPC,PATR,PATRC,SUMSD,RMSD)

      WRITE(*,200) sumsd,lsum
      WRITE(*,300) rmsd,lrms
      WRITE(KC,200) SUMSD,LSUM
      WRITE(KC,300) RMSD,LRMS

      IF(SUMSD.LT.LSUM.and.RMSD.LT.LRMS.and.wncritic.eq.0.) GOTO 109
      IF(SUMSD.LT.fsum1) wncritic=0.0


      IF (INCB .GT.1) THEN
          IF(OUTC) THEN
              READ (INCHAN,*,END=99,ERR=99) KNAME,NNAME,pATRCNAM
              WRITE(*,*)' WRITING 3 FILES......',KNAME,NNAME,pATRCNAM

              xe1=xs-dx*float(NTOT-1)
            CALL WRITESC(KNAME,IMAGK,NTOT,XS,XE1,XT,YT,RES,COMM,IER)
            CALL WRITESC(pATRCNAM,PATRC,Nfix,XS,XE, XT,YT,RES,COMM,IER)
            CALL WRITESC(NNAME,REALN,Nfix,XS,XE,XT,YT,RES,COMM,IER)
          ENDIF
      ENDIF

C     Write k,n and calculated pATR every 5th. cycle, overwriting.
99     IF (INCB/5*5.EQ.INCB) THEN
       WRITE(*,*)' WRITING 3 FILES...k5th.spc/n5th.spc/pATR5th.spc'
       write(KC,*)' WRITING 3 FILES...k5th.spc/n5th.spc/pATR5th.spc'
       CALL WRITESC(' K5TH.spc',IMAGK,Nfix,XS,XE,XT,YT,RES,COMM,IER)
       CALL WRITESC('PATR5TH.spc',PATRC,Nfix,XS,XE,XT,YT,RES,COMM,IER)
       CALL WRITESC(' N5TH.spc',REALN,Nfix,XS,XE,XT,YT,RES,COMM,IER)
      ENDIF

C   Calculate new phase shift spectrum from the n and k spectra
      CALL NK2PH(REALN,IMAGK,NROD,PHASE,NSPC)
 101  CONTINUE

C      THIS IS THE END OF THE REFINEMENT LOOP


      WRITE(*,*)' CONVERGENCE LIMIT HAS NOT BEEN REACHED!'
      WRITE(KC,*)' CONVERGENCE LIMIT HAS NOT BEEN REACHED!'
      WRITE(*,*)' WRITING THE FINAL n, k, pATR AND PHASE SHIFT FILES'
      GOTO 119

109   WRITE(*,*)' ITERATION CONVERGES AT CYCLE:',INCB
      WRITE(KC,*)' ITERATION CONVERGES AT CYCLE:',INCB
      WRITE(*,*)' WRITING FITTED RESULTS ......'
      WRITE(KC,*)' ITERATION SUCCEEDED!'

119   CONTINUE
      CALL WRITESC(Kfinal,IMAGK,Nfix,XS,XE,XT,YT,RES,COMM,IER)
      CALL WRITESC(Nfinal,REALN,Nfix,XS,XE,XT,YT,RES,COMM,IER)
      CALL WRITESC(pATRCfin,PATRC    ,Nfix,XS,XE,XT,YT,RES,COMM,IER)     
      CALL WRITESC(PHASEfin,PHASE    ,Nfix,XS,XE,XT,YT,RES,COMM,IER)

100   FORMAT(1X,' File ',A12,' contains ',I4,' pts, starting at ',
     + F6.1,', ending at ',F6.1)
200   format(1x,' SUM SQUARED DEVIATION:',6p,F11.1,
     +           'E-6     Limit:',6p,F5.1,'E-6')
300   format(1x,' RMS PERCENT DEVIATION:',3p,F11.1,
     +           'E-3     Limit:',F5.1,'E-3')
500   CALL PUTCHAN(INCHAN,IER)
      CALL PUTCHAN(KC,IER)
      END

      SUBROUTINE MAKENROD(NAME,NROD)
C	This version of MAKENROD was modified by JEB April 2000
      INTEGER*1     XT,YT
	INTEGER*2	  KC,RODCHAN
      INTEGER*4     NSPC,IER
      CHARACTER*3   NRODSPC,BIGSPC
      CHARACTER*12  NAME,EXTND4KK,BIGNAME,APNDNAME
      CHARACTER*192 COMM
      REAL*4        NROD(*),LWN,LNR
      DIMENSION     WN(100),RN(100)
	
      COMMON/JEB1/  NSPC,XS,XE,RES,iread,NTOT
      COMMON/JEB3/XT,YT
      COMMON/CHAR/EXTND4KK,COMM,NRODSPC,BIGSPC,BIGNAME,APNDNAME
	COMMON/textIO/ KC

      CALL GETCHAN(RODCHAN,IER)
      OPEN(RODCHAN,FILE=NAME,STATUS='OLD')
      DELTAF=(XE-XS)/FLOAT(NSPC-1)
      READ(RODCHAN,*,END=10) (WN(I),RN(I),I=1,100)
10    CALL PUTCHAN(RODCHAN,IER)


      DO 20 I=1,100
      IF(XS.LT.WN(I)) GOTO 20
      FWN=WN(I-1)
      LWN=WN(I)
      FNR=RN(I-1)
      LNR=RN(I)
      IN=I
      GOTO 30
20    CONTINUE

30    H=(FNR-LNR)/(FWN-LWN)
      J=0
      X=XS
40    CONTINUE
      IF(X.LT.LWN) THEN
      IN=IN+1
      FWN=LWN
      LWN=WN(IN)
      FNR=LNR
      LNR=RN(IN)
      H=(FNR-LNR)/(FWN-LWN)
      ELSE
      J=J+1
      NROD(J)=LNR+H*(X-LWN)
      IF(X.LT.XE) GO TO 50
      X=Xs+DELTAF*float(j)
      GO TO 40
      ENDIF
      IF(X.GT.XE) GO TO 40
50     CONTINUE
C	The following is to output an NROD.SPC file if desired.
      IF (NRODSPC.NE.'YES') GO TO 60
      CALL WRITESC('    nROD.SPC',NROD,NSPC,XS,XE,XT,YT,RES,COMM,IER)
      WRITE(KC,*)' nROD.SPC written'
      WRITE(*,*)'MAKENROD: RETURNING TO MAIN; nrod(1) =',nrod(1),
     +'nrod(NSPC) = ',nrod(NSPC)
60      CONTINUE
      RETURN
      END

      SUBROUTINE pATR2PH(PATR,NROD,IMAGK,PH1,X,X2,RNSNMAX)
C     To calculate the s-polarized phase shift spectrum from an
C     experimental CIRCLE Cell spectrum of an isotropic sample with
C     NREF effective reflections
C
C      In the CIRCLE Cell the angle of incidence is 45 deg and the reflectance
C      is 0.5*(Rs**REF + Rp**REF) = 0.5*(Rs**REF + Rs**2REF), where
C      REF is the effective number of reflections
      INTEGER*4   NPTS
      REAL*4      NROD(*),IMAGK(*)
      DIMENSION   PH1(*),X(*),X2(*),PATR(*)
      COMMON/JEB1/  NPTS,XS,XE,RES,iread,NTOT
	COMMON/JEB2/  WNCRITIC,NREST,RLAST,RNINF,REF
  
C     CALCULATE Rs FROM EXPERIMENTAL pATR;
C     M=EXPERIMENTAL ATR=10**(-pATR)=(Rs**m + Rs**2m)/2, therefore
C     Rs**m=(SQRT(1+8M)-1)/2
C     USE IMAGK TO STORE 0.5*Ln(Rs) TEMPORARILY.
      WRITE(*,*)'  SubR PATR2PH: Getting 0.5*LnRs from pATR'
      DO 20 I=1,NPTS
      RSM=ABS(0.5*(SQRT(1.0+8.0*10.0**(-PATR(I)))-1.0))
      Rs=RSM**(1.0/REF)
      IMAGK(I)=0.5*LOG(Rs)
   20 CONTINUE
      WRITE(*,*)'  SubR PATR2NK: Calling K-K TRANSFORM'
C     THE FOLLOWING K-K TRANSFORMATION IS TO GET THE PHASE FROM REFLECTIVITY
      CALL LZKKTSUB(0.0,WNCRITIC,NREST,RLAST,IMAGK,PH1,x,x2)

C     Change sign of results
      Do 10, I=1,NPTS
          PH1(I)=-PH1(I)
   10 Continue
C
C     THE FOLLOWING CORRECTS THE PHASES WITH NROD WAVENUMBER-DEPENDENT
      WRITE(*,*)'  SubR PATR2PH: Applying phase shift correction'
      ph11=ph1(1)
      DO 80 I=1,NPTS
C     THE NEXT CODE LINE IS VALID ONLY FOR 45 deg INCIDENCE
      DPHASE=2.0*ATAN(SQRT(1.0-2.0*(RNSNMAX/NROD(I))**2))
      dphase=3.141593-dphase
      DPHASE=DPHASE-PH11
      PH1(I)=PH1(I)+DPHASE
 80   CONTINUE
      WRITE(*,*)'  SubR PATR2PH: returning to MAIN'
      RETURN
      END


      SUBROUTINE RPH2K(PATR,NROD,PH1,IMAGK)
C     To calculate the k spectrum of the sample from the 
C     s-polarized reflectance and phase shift spectra from an
C     experimental CIRCLE Cell spectrum of an isotropic sample with
C     NREF effective reflections.
C     In the CIRCLE Cell the angle of incidence is 45 deg and the reflectance
C     is 0.5*(Rs**REF + Rp**REF) = 0.5*(Rs**REF + Rs**2REF), where
C     REF is the effective number of reflections
C
	INTEGER*2	  KC
      INTEGER*4	  NSPC
      REAL*4        NROD(*),IMAGK(*)
      DIMENSION     PATR(*),PH1(*)
      COMMON/JEB1/  NSPC,XS,XE,RES,iread,NTOT
	COMMON/JEB2/  WNCRITIC,NREST,RLAST,RNINF,REF
	COMMON/textIO/ KC

C     CALCULATE k FROM REFLECTIVITY AND PHASE
      C2=1.0/SQRT(2.0)
      WRITE(*,*)'  SubR RPH2K: Calculating k from Rs and THETAs'
C     Do calculation
      DO 120 I=1,NSPC
C     This 'IF' section bypasses points with very small phase shifts
      if(ph1(i).lt.-4.7124)  then
            WRITE(KC,*) 'ph=',ph1(i),' I=',i
            IMAGK(i)=0.0
            go to 120
      end if

C     CALCULATE Rs from experimental pATR.
C     M=EXPERIMENTAL ATR=10**(-pATR)=(Rs**m + Rs**2m)/2, therefore
C     Rs**m=(SQRT(1+8M)-1)/2
      RSM=ABS(0.5*(SQRT(1.0+8.0*10.0**(-PATR(I)))-1.0))
      RSI=RSM**(1.0/REF)

      SP=SQRT(RSI)
      CS=COS(PH1(I))
      SS=SIN(PH1(I))
      AC=(1.0+RSI-2.0*SP*CS)**2
      T=1.0+((1.0-RSI)**2-4.0*RSI*SS**2)/AC
      T=T*0.5
      T2=2.0*SP*(1.0-RSI)*SS/AC
      RNI=SQRT(T+SQRT(T*T+T2*T2))
      RNI=RNI*C2
      IMAGK(I)=T2*nrod(i)/(2.0*RNI)
120   CONTINUE
C      WRITE(*,*)'  SubR RPH2K: returning to MAIN'
      RETURN
      END

      SUBROUTINE NK2PH(REALN,IMAGK,NROD,PHASE1,NP)
C     To calculate the phase shift for a single s-polarized 
C	internal reflection at 45 deg incidence from n and k spectra

      REAL*4    IMAGK(*),NROD(*)
	DIMENSION REALN(*),PHASE1(*)

C     CALCULATE FOR ALL WAVENUMBERS IN INPUT n AND k SPECTRA.
C     THE EQUATIONS USED ARE IN NK2RsPHI.DOC and were coded
C      specifically for 45 deg incidence
      WRITE(*,*) 'SubR NK2PH calculating phase shift from n and k'
      DO I=1,NP
            P2N=REALN(I)*REALN(I)
            P2K=IMAGK(I)*IMAGK(I)
            P2N0=NROD(I)*NROD(I)
C     THE NEXT LINE IS XA=n**2 - k**2 - nrod**2*sin**2*(incident angle)
C     FOR 45 deg INCIDENCE
            XA=P2N-P2K-P2N0/2.0
            P2a=XA*XA
            xb=2.0*REALN(I)*IMAGK(I)
            p2b=xb*xb
            cab=SQRT(P2a+p2b)
	CABPXA=CAB+XA
	CABMXA=CAB-XA
	IF (CABPXA .LT. 0.0)CABPXA=-CABPXA
	IF (CABMXA .LT. 0.0)CABMXA=-CABMXA
            yA=0.70710678*sqrt(CABPXA)
            yB=0.70710678*sqrt(CABMXA)
C     THE NEXT LINE IS WN0=nrod*cos(incident angle) FOR 45 deg INCIDENCE
            WN0=0.70710678*NROD(I)
            CHKCS=YA*YA+YB*YB-WN0*WN0
C      Calculate the phase shift
C     THE CORRECT PHASE IS BETWEEN PI and 0 BECAUSE SIN(PHASE)>0
C     AND COS(PHASE)<0 OR >0 as CHKCS IS <0 OR >0.
C     BELOW,(2.0*WN0*YB/CHKCS) HAS THE SIGN OF CHKCS, AND
C     ATAN(2.0*WN0*YB/CHKCS) IS BETWEEN + and - PI/2.
C     THE FOLLOWING ENSURES THE CORRECT VALUE IS RETURNED.
            IF(ABS(CHKCS).LT.1.0E-7) THEN
                PHASE1(I)=3.141593/2.0
            ELSE
            IF(CHKCS.LT.0.0) THEN
                    PHASE1(I)=3.141593+ATAN(2.0*WN0*YB/CHKCS)
            ELSE
                PHASE1(I)=ATAN(2.0*WN0*YB/CHKCS)
            END IF
            END IF
      END DO
C	WRITE(*,*) 'SubR NK2PH returns to MAIN'
      return
      END

      SUBROUTINE REFLCALN(R,REALN,IMAGK,NROD)
C      THIS SUBROUTINE CALCULATES REFLECTIVITIES AS A FUNCTION OF
C      WAVENUMBERS. IT NEEDS N,K,N(ROD) SPECTRA.

      REAL*4 NROD(*),IMAGK(*),REALN(*),R(*)
      real*8 nrod2,imagk2,x,y,realn2,a,anr
      COMMON/JEB1/  NSPC,XS,XE,RES,iread,NTOT
	COMMON/JEB2/  WNCRITIC,NREST,RLAST,RNINF,REF

      WRITE(*,*)' REFLCALC BEGINS......'
      DO 20 I=1,NSPC
      REALN2=dble(REALN(I))**2
      IMAGK2=dble(IMAGK(I))**2
      NROD2 =dble(NROD(I))**2
      X=REALN2-IMAGK2-NROD2*0.5d0
      Y=dSQRT(X*X+4.d0*REALN2*IMAGK2)
      A=dSQRT(.5d0*(X+Y))
      ANR=A*dble(NROD(I))*dsqrt(2.d0)

      R(I)=1.d0-2.d0*ANR/(NROD2*0.5d0+ANR+Y)
      R(I)=R(I)**REF
20    R(I)=-ALOG10(0.5*R(I)*(1.+R(I)))
      RETURN
      END

      SUBROUTINE STATS(NSPC,PO,PC,SUMSD1,RMSD1)
      REAL*4 PO(*),PC(*)
      real*8 sumsd,rmsd,d,p

      WRITE(*,*)' SubR STATS BEGINS......'
      SUMSD=0.d0
      RMSD=0.d0
      DO 10 I=1,NSPC
        IF(PC(I).LE.1.6E-5.OR.PO(I).LE.1.6E-5) goto 10
        D=dble(PO(I))-dble(PC(I))
        P=D/dble(PO(I))
        SUMSD=SUMSD+D*D
        RMSD=RMSD+P*P
10     CONTINUE
      RMSD=dSQRT(RMSD*(1.d4/dble(NSPC)))
      rmsd1=real(rmsd)
      sumsd1=real(sumsd)
      RETURN
      END


      SUBROUTINE LZKKTSUB(RNINF,XIN,NEXTD,RLAST,RR,RI,x,x2)
      INTEGER*1     XT,YT
	INTEGER*2	  KC
      INTEGER*4     NFIX
      CHARACTER*3   NRODSPC,BIGSPC
      CHARACTER*12  EXTND4KK,BIGNAME,APNDNAME
      CHARACTER*192 COMM
      DIMENSION RR(*),RI(*),X2(*),XK(NPTLIMIT),x(*)
      COMMON/JEB1/  NFIX,XS,XE,RES,iread,NP
      COMMON/JEB3/  XT,YT
      COMMON/CHAR/  EXTND4KK,COMM,NRODSPC,BIGSPC,BIGNAME,APNDNAME
	COMMON/textIO/ KC
	COMMON/FOR_READ/NPTLIMIT

C     NPTLIMIT=9336 points holds a spectrum between 9000 and 0 CM-1 
C     with spacing 0.964 cm-1
      IF (NFIX.LE.9336) GOTO 5
      STOP ' LZKKTSUB: STOP -- TOO MANY POINTS'

   5    H=(Xs-Xe)/FLOAT(Nfix-1)
C       Check if approximate method wanted. Proceed if yes or bypass
      IF (XIN.EQ.0.0) GOTO 8
      WRITE(KC,*)' LZKKTSUB: Setting up approximate KK above ',XIN,
     +' cm-1'
C       make NL an even number (simpler to program)
      FNL=ABS((XIN-XS)/H)
      NL=IFIX(FNL)/2
      NL=NL*2

8     iread=iread+1
      if(iread.ne.1.and.nextd.lt.0) goto 300
      np=nfix
      nmax=IFIX(ABS(xe/H))+1
      XE1=XE

      if (nextd) 100, 300, 200

c       reading spectrum from disk
100   WRITE(KC,*)' LZKKTSUB: Appending spectrum below ',XE1,' cm-1'
      call appendspc(rr,np,nmax,H,XE1)
C     After appendspc (i.e. first time through) write appended 
C     spectrum to disk
	IF (BIGSPC .EQ. 'YES') 
     +CALL WRITESC(APNDNAME,RR,NP,XS,XE1,XT,YT,RES,COMM,IER)
      WRITE(KC,*)' LZKKTSUB: After Appendspc,KK input written 
     +to ', APNDNAME
c       adding a straight line
200   call appendline(nextd,rlast,rr,np,nmax,H,XE1)
      do i=nfix+1,np
            xk(i)=rr(i)*x(i)
      end do

300   continue

C	Each time through,write extended file to disk
      IF (BIGSPC .NE. 'YES') Go To 20
      CALL WRITESC(BIGNAME,RR,NP,XS,XE1,XT,YT,RES,COMM,ier)
      WRITE(KC,*)' LZKKTSUB: Extended KK input written to ',BIGNAME

20    NPTS=NFIX
      do i=1,nfix
            xk(i)=rr(i)*X(I)
      end do

      IF (XIN.EQ.0.0.OR.NL.LE.200) THEN
            CALL KKT(H,NPTS,NP,RI,X2,XK)
      ELSE
            CALL KKTM(NL,H,NPTS,NP,RI,X2,XK)
      ENDIF

      DO 22 I=1,NFIX
22      RI(I)=RNINF+RI(I)
      WRITE(*,*)' SUBROUTINE LZKKTSUB returns'
      END

      SUBROUTINE APPENDSPC(imagk,nnew,nmax,dx,XE1)
C	N.B. This version of appendline was modified by JEB, April 2000
      real k1,imagk
      integer*1 xt,yt
      INTEGER*2 KC
      INTEGER*4 n1,NFIX
      CHARACTER*12 EXTND4KK
      CHARACTER*30 Unused
      character*192 comm
      dimension k1(4096), imagk(*)
      COMMON/JEB1/NFIX,XS,XE,RES,iread,NTOT
      COMMON/CHAR/EXTND4KK,COMM,Unused
	COMMON/textIO/ KC

c       read extension spectrum from disk
      call readsc(EXTND4KK,k1,n1,xs1,xe1,xt,yt,res,comm,ier)
      WRITE(KC,*)' SubR Appendspc: Spectrum read from ',EXTND4KK

      dx1=(xs1-xe1)/float(n1-1)
      ispace=(abs(dx)+0.001)/abs(dx1)

c       if no overlap between the original spectrum and the extra k
      if((xe-xs1).gt.(-abs(dx*1.01))) then
            write(*,*)' The extra k spectrum is NOT overlapped',
     +                    ' with the original ! Check and re-run.'
            WRITE(KC,*)' The extra k spectrum is NOT overlapped',
     +                    ' with the original ! Check and re-run.'
            CLOSE(KC)
            stop ' '
c       if the spectrum read from disk has bigger spacing
      elseif(ispace.lt.1) then
		call msg('\  \')
		call msg('\ The extra k spectrum has bigger spacing.\')  
		call msg('\ It needs to be interpolated.\')
		call msg('\ Do interpolation in SpectraCalc and\') 
		call msg('\    re-start the refining !\')
		WRITE(KC,*)' The extra k spectrum has a bigger spacing.'
     +                   ,' It needs to be interpolated.'
		WRITE(KC,*)' Do interpolation in SpectraCalc and re-',
     +                    'start the refining.'
		CLOSE(KC)
		stop '  '
c       if the extra k is O.K.
      else
            xstart=xe-dx
            xlimit=dx/100.0
            do i=1,n1
               xtemp=xs1-dx1*float(i-1)
               if(abs(xtemp-xstart).lt.xlimit) goto 10
            end do
10              nstart=i

c       if the extra k spectrum is offset from the original spectrum
            if (abs(xtemp-xe1).lt.xlimit) then
                  write(*,*)' The extra k is offset in cm-1',
     +                            ' from the original spectrum!'
			write(*,*)' Do interpolation of 2*N and ',
     +                  'truncate odd # of points at the ',
     +                  'high cm-1 side.'
			WRITE(KC,*)' The extra k is offset in cm-1',
     +                            ' from the original spectrum!'
			WRITE(KC,*)' Do interpolation of 2*N and ',
     +                  'truncate odd # of points at the ',
     +                  'high cm-1 side.'
			CLOSE(KC)
			stop ' '
            endif
            nadd=(n1-nstart+1)/ispace
            if((nfix+nadd).gt.9336) nadd=9336-nfix
		if (nadd.gt.nmax) nadd=nmax
		do i=1,nadd
			imagk(nfix+i)=k1(nstart+(i-1)*ispace)
		end do
		nnew=nnew+nadd
	    NTOT=nnew
	    xe1=XS-dx*float(nnew-1)
      WRITE(KC,*)'  After Appendspc ', nnew,' points, lowest at ',xe1,
     +' cm-1'
      WRITE(*,*)'  SubR Appendspc returns'
      endif
      end

      SUBROUTINE APPENDLINE(nextd,vlast,imagk,nnew,nmax,dx,XE1)
C	N.B. This version of appendline was modified by JEB, May 2000
      real imagk
	INTEGER*2 KC
      INTEGER*4 NFIX
      dimension imagk(*)
      COMMON/JEB1/NFIX,XS,XE,RES,iread,NTOT
	COMMON/textIO/ KC

      if (nnew.ge.9336) return
      ndd=iabs(nextd)
      if (ndd.gt.nmax) ndd=nmax
      ndd=ndd-(nnew-nfix)
      if (ndd.le.0) return

      WRITE(KC,*)' APPENDLINE: Adding straight line below ',XE1,' cm-1'
      nextdt=nnew+ndd
      if (nextdt.gt.9336) nextdt=9336
      dk=(imagk(nnew)-vlast)/float(nextdt-nnew)

      do i=nnew+1,nextdt
            imagk(i)=imagk(nnew)-dk*float(i-nnew)
      end do
      nnew=nextdt
      NTOT=nnew
      xe1=XS-dx*float(nnew-1)

      WRITE(KC,*)' SubR Appendline: In KK transform ', nnew,' points,
     + lowest at ',xe1,' cm-1'
      WRITE(*,*)'  SubR Appendline returns'
      END

      SUBROUTINE KKTM(NL,H,NFIX,N,RN,X2,XK)
C     -- SAME AS KKT EXCEPT USING THE APPROXIMATION
	INTEGER*2 KC
      DIMENSION RN(*),X2(*),XK(*)
	COMMON/textIO/ KC

      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,923) IH,IM,IS,is100
      WRITE(KC,923) IH,IM,IS,IS100

      HPI=1.2732395447D0*ABS(H)

C     FIRST PART OF THE WHOLE RANGE
      CALL KSUM(2,NL,100,1,N,X2,XK,RN,HPI)

C     SECOND PART OF THE WHOLE RANGE
      N2=NL+200
      CALL KSUM(NL,N2,2,1,N,X2,XK,RN,HPI)
      N1=NL+1
      CALL KSUM(N1,N2,2,2,N,X2,XK,RN,HPI)

C     THIRD PART OF THE WHOLE RANGE
      N1=N2+1
      N2=NFIX
      IF (NFIX.EQ.N) N2=N2-1
      CALL KSUM(N1,N2,2,NL,N,X2,XK,RN,HPI)
      N1=N1+1
      N4=NL-1
      CALL KSUM(N1,N2,2,N4,N,X2,XK,RN,HPI)

C     INTERPOLATING FOR 1ST PART
      NI=(NL-2)/100
      DO 410 I=1,NI
      I2=2+I*100
      I1=I2-100
      SLOPE=RN(I2)-RN(I1)
      I21=I2-1
      I11=I1+1
      DO 410 J=I11,I21
410   RN(J)=RN(I1)+SLOPE*(J-I1)/100.
      I1=2+NI*100
      SLOPE=RN(NL)-RN(I1)
      I21=NL-1
      I11=I1+1
      DO 420 J=I11,I21
420   RN(J)=RN(I1)+SLOPE*(J-I1)/(NL-I1)

      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,924) IH,IM,IS,is100
      WRITE(KC,924) IH,IM,IS,is100
923   FORMAT('  BEFORE KK, TIME IS ',I2,':',I2.2,':',I2.2,'.',i2.2,
     1     /,'  DOING TRANSFORM...KKTM...PLEASE WAIT')
924   FORMAT('  AFTER  KK, TIME IS ',I2,':',I2.2,':',I2.2,'.',i2.2)
      RN(1)=RN(2)
      IF (NFIX.EQ.N) RN(NFIX)=RN(NFIX-1)
      RETURN
      END

      SUBROUTINE KKT(H,NFIX,N,RN,X2,XK)
	INTEGER*2 KC
      DIMENSION RN(*),X2(*),XK(*)
	COMMON/textIO/ KC

      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,923) IH,IM,IS,is100
      WRITE(KC,923) IH,IM,IS,IS100
      HPI=1.2732395447D0*ABS(H)

      N2=NFIX
      IF (NFIX.EQ.N) N2=N2-1
      CALL KSUM(2,N2,2,1,N,X2,XK,RN,HPI)
      CALL KSUM(3,N2,2,2,N,X2,XK,RN,HPI)

      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,924) IH,IM,IS,is100
      WRITE(KC,924) IH,IM,IS,is100
923   FORMAT('  BEFORE KK, TIME IS ',I2,':',I2.2,':',I2.2,'.',i2.2,
     1     /,'  DOING TRANSFORM...KKT...PLEASE WAIT')
924   FORMAT('  AFTER  KK, TIME IS ',I2,':',I2.2,':',I2.2,'.',i2.2)
      IF (NFIX.EQ.N) RN(NFIX)=RN(NFIX-1)
      RN(1)=RN(2)
      RETURN
      END

      SUBROUTINE KSUM(N1,N2,N3,N4,N5,X2,XK,RN,HPI)
      DIMENSION RN(*),X2(*),XK(*)

      DO 130 I=N1,N2,N3
      S=0.0
      DO 125 J=N4,N5,2
125   S=S+XK(J)/(X2(J)-X2(I))
130   RN(I)=HPI*S
      RETURN
      END
